# w.c.s. - web application for online forms
# Copyright (C) 2005-2024  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from quixote import get_publisher, get_request

from wcs.api_utils import get_query_flag
from wcs.qommon import _


class PickableClamD:
    @property
    def clamd(self):
        if getattr(self, '_clamd', None) is None:
            self._clamd = {}
        return self._clamd

    def init_clamd(self):
        if get_publisher().has_site_option('enable-clamd') and 'waiting_scan' not in self.clamd:
            self.clamd['waiting_scan'] = True

    def force_not_malware(self):
        self.clamd['waiting_scan'] = False
        self.clamd['returncode'] = 100  # forced-ok

    def is_waiting_for_scan(self):
        if not get_publisher().has_site_option('enable-clamd'):
            return False
        return self.clamd.get('waiting_scan', False)

    def has_malware(self):
        if not get_publisher().has_site_option('enable-clamd'):
            return False
        if 'waiting_scan' not in self.clamd:
            # files uploaded before clamd was enabled
            return False
        if self.clamd['waiting_scan']:
            # file not scanned yet
            return False
        return bool(self.clamd['returncode'] == 1)

    def allow_download(self, formdata=None):
        if (
            formdata
            and get_request()
            and formdata.is_submitter(get_request().user)
            and not get_request().is_in_backoffice()
        ):
            # always allow submitter to download its file
            return True
        if get_query_flag('force-download') and get_request().user and get_request().user.is_admin:
            # allow admin user to force download
            return True
        return not (self.is_waiting_for_scan() or self.has_malware())

    def get_view_clamd_status(self):
        if get_publisher().has_site_option('enable-clamd'):
            if self.has_malware():
                return ' <span class="malware-file">%s</span>' % _('A malware was found in this file.')
            if self.is_waiting_for_scan():
                return ' <span class="waiting-for-scan-file">%s</span>' % _(
                    'The file is waiting to be checked for malware.'
                )
        return ''
